@@ -588,7 +588,7 @@ void do_get(char *rname,char *lname)
 	  setup_pkt(outbuf);
 
 	  SSVAL(outbuf,smb_vwv0,fnum);
-	  SSVAL(outbuf,smb_vwv1,MIN(1024*((max_xmit-1024)/1024),finfo.size - nread));
+	  SSVAL(outbuf,smb_vwv1,MIN(max_xmit-200,finfo.size - nread));
 	  SSVAL(outbuf,smb_vwv2,nread & 0xFFFF);
 	  SSVAL(outbuf,smb_vwv3,nread >> 16);
 	  SSVAL(outbuf,smb_vwv4,finfo.size - nread);
@@ -613,6 +613,11 @@ void do_get(char *rname,char *lname)
 	  }
       
       nread += datalen;
+      if (nread == 0) 
+	{
+	  Debug(0,"Error reading file %s. Got 0 bytes\n",rname);
+	  break;
+	}
     }
 
 
@@ -623,8 +628,7 @@ void do_get(char *rname,char *lname)
   setup_pkt(outbuf);
 
   SSVAL(outbuf,smb_vwv0,fnum);
-  SSVAL(outbuf,smb_vwv1,0);
-  SSVAL(outbuf,smb_vwv2,0);
+  SIVAL(outbuf,smb_vwv1,finfo.mtime);
 
   send_smb(outbuf);
   receive_smb(inbuf);
@@ -1182,7 +1186,7 @@ void do_put(char *rname,char *lname,file_info *finfo)
   
   while (nread < finfo->size)
     {
-      int n = MIN(1024*((max_xmit-1024)/1024),finfo->size - nread);
+      int n = MIN(max_xmit-200,finfo->size - nread);
   
       memset(outbuf,0,smb_size);
       set_message(outbuf,5,n + 3);
