@@ -61,6 +61,7 @@ BOOL daemon = False;
 
 
 BOOL got_bcast = False;
+BOOL got_nmask = False;
 
 /****************************************************************************
 add a netbios name
@@ -552,8 +553,16 @@ BOOL init_structs(void )
   memcpy(&myip,hp->h_addr,4);
 
   /* Read the broadcast address from the interface */
-  if (!got_bcast)
-    get_broadcast(&myip,&bcast_ip,&Netmask);
+  {
+    struct in_addr ip1,ip2;
+    get_broadcast(&myip,&ip1,&ip2);
+
+    if (!got_bcast)
+      bcast_ip = ip1;
+    
+    if (!got_nmask)
+      Netmask = ip2;    
+  }
 
   if (*myname == 0)
     strcpy(myname,myhostname);
@@ -574,6 +583,7 @@ void usage(char *pname)
   printf("\t-l log basename.      Basename for log/debug files\n");
   printf("\t-n netbiosname.       the netbios name to advertise for this host\n");
   printf("\t-B broadcast address  the address to use for broadcasts\n");
+  printf("\t-N netmask           the netmask to use for subnet determination\n");
   printf("\t-L name              lookup this netbios name then exit\n");
   printf("\t-S                   serve queries via DNS if not on the same subnet\n");
   printf("\n");
@@ -592,7 +602,7 @@ int main(int argc,char *argv[])
   extern char *optarg;
   pstring lookup = "";
 
-  while ((opt = getopt (argc, argv, "L:B:Rn:l:d:Dp:hPS")) != EOF)
+  while ((opt = getopt (argc, argv, "L:B:N:Rn:l:d:Dp:hPS")) != EOF)
     switch (opt)
       {
       case 'B':
@@ -602,6 +612,13 @@ int main(int argc,char *argv[])
 	  got_bcast = True;
 	}
 	break;
+      case 'N':
+	{
+	  unsigned long a = inet_addr(optarg);
+	  memcpy(&Netmask,&a,sizeof(a));
+	  got_nmask = True;
+	}
+	break;
       case 'n':
 	strcpy(myname,optarg);
 	break;
