@@ -43,7 +43,6 @@ int Protocol = PROT_CORE;
 
 int maxxmit = BUFFER_SIZE;
 
-char *original_inbuf = NULL;
 int chain_size = 0;
 
 /* a fnum to use when chaining */
@@ -796,7 +795,7 @@ BOOL open_sockets(BOOL daemon,int port)
 	}
   
       /* now we've got a socket - we need to bind it */
-      while (bind(s, (struct sockaddr * ) &sock,sizeof(sock)) < 0) 
+      if (bind(s, (struct sockaddr * ) &sock,sizeof(sock)) < 0)
 	{
 	  perror("bind");
 	  close(s);
@@ -832,6 +831,8 @@ BOOL open_sockets(BOOL daemon,int port)
 	  if (Client != -1 && fork()==0)
 	    {
 	      signal(SIGPIPE, SIGNAL_CAST abort);
+	      signal(SIGCLD, SIGNAL_CAST SIG_DFL);
+
 	      /* now set appropriate socket options */
 	      {
 		int one=1;
@@ -1124,7 +1125,7 @@ int reply_connect_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   SSVAL(outbuf,smb_tid,connection_num);
 
   CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,outsize-4);
+  SSVAL(outbuf,smb_vwv1,(chain_size + outsize)-4);
 
   strcpy(smb_buf(outbuf),"A:");
 
@@ -1196,7 +1197,7 @@ int reply_sesssetup(char *inbuf,char *outbuf,int length,int bufsize)
   outsize = set_message(outbuf,3,0);
 
   CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,outsize-4);
+  SSVAL(outbuf,smb_vwv1,(chain_size+outsize)-4);
 
   if (guest)
     SSVAL(outbuf,smb_vwv2,1);
@@ -1833,7 +1834,7 @@ int reply_open_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   
   outsize = set_message(outbuf,15,0);
   CVAL(outbuf,smb_vwv0) = smb_com2;
-  SSVAL(outbuf,smb_vwv1,outsize-4);
+  SSVAL(outbuf,smb_vwv1,(chain_size+outsize)-4);
   SSVAL(outbuf,smb_vwv2,fnum);
   SSVAL(outbuf,smb_vwv3,fmode);
   SIVAL(outbuf,smb_vwv4,mtime);
@@ -2284,7 +2285,7 @@ int reply_read_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   CVAL(outbuf,smb_vwv0) = smb_com2;
   SSVAL(outbuf,smb_vwv1,(outsize+chain_size)-4);
   SSVAL(outbuf,smb_vwv5,nread);
-  SSVAL(outbuf,smb_vwv6,(int)(data - (inbuf+4)));
+  SSVAL(outbuf,smb_vwv6,(int)(data - ((outbuf-chain_size)+4)));
   SSVAL(smb_buf(outbuf),-2,nread);
   
   Debug(2,"%s readX fnum=%d cnum=%d min=%d max=%d nread=%d com2=%d off2=%d\n",
@@ -2294,7 +2295,7 @@ int reply_read_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   chain_fnum = smb_fid;
 
   if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,original_inbuf+smb_off2+4,
+    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
 			   outbuf,outbuf+outsize,
 			   length,bufsize);
   
@@ -2546,7 +2547,7 @@ int reply_write_and_X(char *inbuf,char *outbuf,int length,int bufsize)
   chain_fnum = fnum;
 
   if (smb_com2 != 0xFF)
-    outsize += chain_reply(smb_com2,inbuf,original_inbuf+smb_off2+4,
+    outsize += chain_reply(smb_com2,inbuf,inbuf+smb_off2+4,
 			   outbuf,outbuf+outsize,
 			   length,bufsize);
   
@@ -3454,25 +3455,30 @@ int chain_reply(int type,char *inbuf,char *inbuf2,char *outbuf,char *outbuf2,int
 {
   int outsize;
   char *ibuf,*obuf;
+  static int chain_size_in = 0;
   static BOOL in_chain = False;
   static char *last_inbuf=NULL,*last_outbuf=NULL;
-  BOOL was_inchain;
-
+  BOOL was_inchain = in_chain;
 
   chain_size += ((int)outbuf2 - (int)outbuf) - smb_wct;
-
-  was_inchain = in_chain;
-
-  if (in_chain)
+  if (was_inchain)
     {
       inbuf = last_inbuf;
       outbuf = last_outbuf;
     }
+  else
+    chain_size_in = 0;
 
   in_chain = True;
   last_inbuf = inbuf;
   last_outbuf = outbuf;
 
+
+  inbuf2 -= chain_size_in;
+
+  chain_size_in += ((int)inbuf2 - (int)inbuf) - smb_wct;
+
+
   /* allocate some space for the in and out buffers of the chained message */
   ibuf = (char *)malloc(size);
   obuf = (char *)malloc(bufsize);
@@ -3520,7 +3526,7 @@ int chain_reply(int type,char *inbuf,char *inbuf2,char *outbuf,char *outbuf2,int
   if (ibuf) free(ibuf);
   if (obuf) free(obuf);
 
-  in_chain = False;
+  in_chain = was_inchain;
 
   /* return how much extra has been added to the packet */
   return(outsize - smb_wct);
@@ -3537,7 +3543,6 @@ int construct_reply(char *inbuf,char *outbuf,int size,int bufsize)
   int outsize;
   int msg_type = CVAL(inbuf,0);
 
-  original_inbuf = inbuf;
   chain_size = 0;
   
   if (msg_type != 0)
