@@ -173,21 +173,26 @@ int TimeDiff(void)
   if (!initialised)
     {
       time_t t=time(NULL);
+
+/* There are three ways of getting the time difference between GMT and
+   local time. Use the following defines to decide which your system
+   can handle */
 #ifdef HAVE_TIMELOCAL
       timediff = timelocal(gmtime(&t)) - t;
 #else
-      /* call localtime to prime the timezone variable */
+#ifdef HAVE_TIMEZONE
       localtime(&t);
       timediff = timezone;
-      Debug(3,"timediff=%d\n",timediff);
+#else
+      timediff = localtime(&t)->tm_gmtoff;
 #endif
+#endif
+      Debug(3,"timediff=%d\n",timediff);
       initialised = True;
     }
 
 return(timediff);
 }
-  
-
 
 /****************************************************************************
 try to optimise the timelocal call, it can be quite expenive on some machines
@@ -288,8 +293,10 @@ while(*in)
 /****************************************************************************
 mangle a name into netbios format
 ****************************************************************************/
-void name_mangle(unsigned char *in,unsigned char *out)
+void name_mangle(char *In,char *Out)
 {
+  unsigned char *in = (unsigned char *)In;
+  unsigned char *out = (unsigned char *)Out;
   int len = 2*strlen((char *)in);
   int pad = 0;
 
@@ -774,9 +781,9 @@ return the absolute current directory path
 char *GetWd(char *s)
 {
 #if (defined(LINUX) || defined(SOLARIS) || defined(SVR4) || defined(HPUX))
-  return(getcwd(s,sizeof(pstring)));
+  return((char *)getcwd(s,sizeof(pstring)));
 #else
-  return(getwd(s));
+  return((char *)getwd(s));
 #endif
 }
 
@@ -2009,7 +2016,7 @@ BOOL get_broadcast(struct in_addr *if_ipaddr, struct in_addr *if_bcast, struct i
       Debug(0,"SIOCGIFBRDADDR failed\n");
       return(False);
     }
-  memcpy(if_bcast,&(*(struct sockaddr_in *) &ifr->ifr_broadaddr).sin_addr,4);
+  *if_bcast = ((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr;
 
 
   /* Get the netmask address from the kernel */
@@ -2018,7 +2025,7 @@ BOOL get_broadcast(struct in_addr *if_ipaddr, struct in_addr *if_bcast, struct i
       Debug(0,"SIOCGIFNETMASK failed\n");
       return(False);
     }
-  memcpy(if_nmask,&(*(struct sockaddr_in *) &ifr->ifr_netmask).sin_addr,4);
+  *if_nmask = ((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr;
   
   /* Close up shop */
   (void) close(sock);
@@ -2487,3 +2494,52 @@ BOOL set_filetime(char *fname,time_t mtime)
 
   return(utime(fname,&times) == 0);
 }
+
+
+#ifdef NOSTRDUP
+/****************************************************************************
+duplicate a string
+****************************************************************************/
+char *strdup(char *s)
+{
+  char *ret = NULL;
+  if (!s) return(NULL);
+  ret = (char *)malloc(strlen(s)+1);
+  if (!ret) return(NULL);
+  strcpy(ret,s);
+  return(ret);
+}
+#endif
+
+
+int System(const char *command)
+{
+  union wait pstat;
+  pid_t pid;
+  int omask;
+  int intsave, quitsave;
+  
+  if (!command)           /* just checking... */
+    return(1);
+  
+  omask = sigblock(sigmask(SIGCHLD));
+  switch(pid = vfork()) {
+  case -1:                        /* error */
+    (void)sigsetmask(omask);
+    pstat.w_status = 0;
+    pstat.w_retcode = 127;
+    return(pstat.w_status);
+  case 0:                         /* child */
+    (void)sigsetmask(omask);
+    execl("/bin/sh", "sh", "-c", command, (char *)NULL);
+    _exit(127);
+  }
+  intsave = signal(SIGINT, SIG_IGN);
+  quitsave = signal(SIGQUIT, SIG_IGN);
+  pid = waitpid(pid, (int *)&pstat, 0);
+  
+  (void)sigsetmask(omask);
+  (void)signal(SIGINT, intsave);
+  (void)signal(SIGQUIT, quitsave);
+  return(pid == -1 ? -1 : pstat.w_status);
+}
